module IpWhitelist
  class List < ActiveRecord::Base
    self.table_name = 'ip_whitelist_lists'
    
    REDIS_KEY = '`ip-whitelist-redis`'

    validates_presence_of :ips, :appid

    after_save :load
    after_destroy :load

    def redis_key
      self.class.redis_key(appid)
    end

    private

    def load
      self.class.load
    end
    
    class << self
      def redis_key appid
        "#{REDIS_KEY}-#{appid}"
      end

      def auth! appid, ip
        unless IpWhitelist.redis.hget(redis_key(appid), ip)
          raise IpWhitelist::Error.new(status: 403, message: 'Invaild Request')
        end
      end

      def load
        redis = IpWhitelist.redis
        redis.keys(redis_key('*')).each { |key| redis.del(key) }
        IpWhitelist::List.all.each do |list|
          list.ips.split(',').each { |ip| redis.hset(list.redis_key, ip, true) }
        end
      end
    end
  end
end
